import re

PATTERNS = {
    "emoji": re.compile(r"<(?:emoji-id|em):(\d+)>(.*?)</(?:emoji-id|em)>", re.DOTALL),
    "mention": re.compile(r"<(?:mention|mo):(\d+)>(.*?)</(?:mention|mo)>", re.DOTALL),
    "link": re.compile(r"<(?:link|url)(?::([^>]+))?>(.*?)</(?:link|url)>", re.DOTALL),
    "spoiler": re.compile(r"<(?:spoiler|sp)>(.*?)</(?:spoiler|sp)>", re.DOTALL),
    "blockquote": re.compile(
        r"<(?:blockquote|bq)(?: hide)?>(.*?)</(?:blockquote|bq)>", re.DOTALL
    ),
}
